package cn.com.ty.lift.enterprise.common;

import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * @author bieao
 * @date 2019/12/3 10:30 AM
 * @description 日志切面
 */
@Slf4j
@Aspect
@Component
public class ControllerAspect {

    private static final String head = "##########|\t";

    @Pointcut("execution(* cn.com.ty.lift.enterprise.*.controller..*(..))")
    public void controllerPointCut() {
    }

    @Before("controllerPointCut()")
    public void doBefore(JoinPoint joinPoint) throws Throwable {
        // Receives the request and get request content
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        log.info(head + "URL : " + request.getRequestURL().toString());
        log.info(head + "HTTP_METHOD : " + request.getMethod());
        log.info(head + "IP : " + request.getRemoteAddr());
        log.info(head + "CLASS_METHOD : " + joinPoint.getSignature().getDeclaringTypeName() + "." + joinPoint.getSignature().getName());
        log.info(head + "ARGS : " + Arrays.toString(joinPoint.getArgs()));

    }

    @AfterReturning(returning = "response", pointcut = "controllerPointCut()")
    public void doAfterReturning(RestResponse response) throws Throwable {
        // Processes the request and returns the content
        log.info(head + "RESPONSE : " + JSONUtil.parse(response));
        log.info("====================================================");
    }

    @AfterThrowing(throwing = "ex", pointcut = "controllerPointCut()")
    public void doAfterThrowing(EnterpriseBasicException ex) throws Throwable {
        final String msg = ex.getMessage();
        // Processes the request and returns the content
        log.info(head + "RESPONSE : " +  ex.addSuffix(msg).replace("()", ""));
        log.info("====================================================");
    }


    @Around("controllerPointCut()")
    public Object interceptor(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        log.info("====================================================");
        return proceedingJoinPoint.proceed();
    }

}
