package cn.com.ty.lift.enterprise.oa.controller;

import cn.com.ty.lift.common.utils.AliyunOSSUtil;
import cn.com.ty.lift.enterprise.common.Conf;
import cn.com.ty.lift.enterprise.common.RestResult;
import cn.com.ty.lift.enterprise.oa.entity.LiftCertificate;
import cn.com.ty.lift.enterprise.oa.service.LiftCertificateService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;


/**
 * 操作证管理
 * Created by Administrator on 2019/11/26.
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("liftCertificate")
public class LiftCertificateController {
    private LiftCertificateService liftCertificateService;

    private AliyunOSSUtil aliyunOSSUtil;

    private Conf conf;

    /**
     * @description 根据id查找
     * @date 2019/12/6 9:45
     * @param
     * @return
     */
    @PostMapping("findOne")
    public RestResponse<LiftCertificate> findOne(Long id) {
        LiftCertificate entity = liftCertificateService.getById(id);
        return RestResult.okData(entity);
    }

    /**
     * @description 根据条件查询
     * @date 2019/12/6 9:45
     * @param
     * @return
     */
    @PostMapping("list")
    public RestResponse<IPage<LiftCertificate>> list(int pageNum, int pageSize, String certificateType, String name, Integer status) {
        if (null == certificateType) {
            certificateType = "1";
        }
        //name 模糊查询
        if (null != name && name.length() > 1) {
            name = "%" + name + "%";
        }
        IPage<LiftCertificate> results = liftCertificateService.selectByTypeCondition(pageNum, pageSize, certificateType, name, status);
        if (results == null) {
            return RestResult.errException();
        }
        return RestResult.okData(results);
    }

    /**
     * @description 更新操作证信息
     * @date 2019/12/4 10:32
     * @param
     * @return
     */
    @PostMapping("modify")
    public RestResponse<Boolean> modify(LiftCertificate entity) {
        if (null == entity) {
            return RestResult.errParam();
        }
        boolean result = liftCertificateService.updateById(entity);
        if (result) {
            return RestResult.okData(result);
        }
        return RestResult.errException();
    }

    /**
     * @description 操作证审核
     * @date 2019/12/4 10:32
     * @param
     * @return
     */
    @PostMapping("audit")
    public RestResponse<Integer> audit(Long id, boolean pass) {
        //0：无证，1：待审核，2：审核未通过，3：审核通过，4：超期
        int status = pass ? 3 : 2;
        int result = liftCertificateService.updateAudit(id, status);
        return RestResult.okData(result);
    }

    /**
     * @description 新增操作证
     * @date 2019/12/4 10:31
     * @param
     * @return
     */
    @PostMapping("add")
    public RestResponse<Object> add(MultipartFile[] files,LiftCertificate entity) throws Exception{
        if(null == files || files.length == 0){
            return RestResult.err("请上传操作证图片");
        }

        if (null == entity) {
            return RestResult.errParam();
        }
        for (int i = 0;i < files.length; i++){
            MultipartFile file = files[i];
            if(file.isEmpty()){
                return RestResult.err("请上传操作证图片");
            }
            String originalFilename = file.getOriginalFilename();
            String suffix = originalFilename.substring(originalFilename.indexOf("."));
            String fileName = IdWorker.getId() + suffix;
            byte[] bytes = file.getBytes();
            aliyunOSSUtil.putObject(conf.bucketName,bytes,fileName);
            if(i == 0){
                entity.setFirstImgUrl(fileName);
            }else{
                entity.setSecondImgUrl(fileName);
            }
        }

        boolean result = liftCertificateService.save(entity);
        return RestResult.okData(result);

    }
}
