package cn.com.ty.lift.system.faq.service.impl;

import cn.com.ty.lift.system.faq.entity.Faq;
import cn.com.ty.lift.system.faq.mapper.FaqMapper;
import cn.com.ty.lift.system.faq.service.FaqService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 常见问题级常见问题分类表 服务实现类
 * </p>
 *
 * @author wang-hai-cheng
 * @since 2019-12-05
 */
@Transactional
@Service
public class FaqServiceImpl extends ServiceImpl<FaqMapper, Faq> implements FaqService {

    public RestResponse getFaqs(Faq faq) {
        List<Faq> faqs = this.list(new QueryWrapper<Faq>()
                .eq(faq.getType() != null, "type", faq.getType())
                .eq(faq.getParentId() != null, "parent_id", faq.getParentId())
                .orderByAsc("serial"));
        if (faqs.isEmpty()) {
            return RestResponse.ok(null, "9", "无数据");
        }
        return RestResponse.ok(faqs, "1", "查询成功");
    }

    /**
     * @description 新增分类或问题或答案
     * @date 2019/12/5 11:01
     */
    public RestResponse add(Faq faq) {
        List<Faq> faqs = this.list(new QueryWrapper<Faq>()
                .select("id", "serial")
                .eq("type", faq.getType()).eq("parent_id", faq.getParentId())
                .orderByDesc("serial"));
        if (faqs.isEmpty()) {
            faq.setSerial(1);
        } else {
            faq.setSerial(faqs.get(0).getSerial() + 1);
        }
        if (this.save(faq)) {
            return RestResponse.ok(null, "1", "新增成功");
        }
        return RestResponse.ok(null, "0", "新增失败");
    }

    public RestResponse delete(Long id) {
        if (deleteAll(id)) {
            return RestResponse.ok(null, "1", "删除成功");
        }
        return RestResponse.ok(null, "0", "删除失败");
    }

    //递归删除id及它下面所有分类，问答
    private boolean deleteAll(Long id) {
        //获取父id下的所有子分类或问答
        List<Faq> faqs = this.list(new QueryWrapper<Faq>().eq("parent_id", id));
        //删除父id下的所有子分类或问答
        this.remove(new QueryWrapper<Faq>().eq("parent_id", id));
        //删除自己
        this.remove(new QueryWrapper<Faq>().eq("id", id));
        //如果父id下还有子分类或问答
        if (faqs.size() != 0) {
            //遍历删除自己及删除自己下的所有子分类或问答
            faqs.forEach(faq -> {
                this.remove(new QueryWrapper<Faq>().eq("parent_id", id));
                this.remove(new QueryWrapper<Faq>().eq("id", id));
                //递归删除
                deleteAll(faq.getParentId());
            });
            //删除完毕返回true
            return true;
        } else {
            //如果没有子分类或问答就直接返回了
            return true;
        }
    }

    //type 1上移 0下移
    public RestResponse serial(Long id, Integer type) {
        Faq byId = this.getById(id);
        if (byId == null) {
            return RestResponse.ok(null, "0", "此id不存在");
        }
        List<Faq> faqs;
        if (type == 1) {
            faqs = this.list(new QueryWrapper<Faq>().eq("type", byId.getType()).eq("parent_id", byId.getParentId()).orderByDesc("serial"));
        } else {
            faqs = this.list(new QueryWrapper<Faq>().eq("type", byId.getType()).eq("parent_id", byId.getParentId()).orderByAsc(("serial")));
        }

        if (faqs.isEmpty()) {
            byId.setSerial(1);
            this.updateById(byId);
            return RestResponse.ok(null, "1", "移动成功");
        }

        Faq faq;
        int i;
        if (type == 1) {
            for (i = 0; i < faqs.size(); i++) {
                if (faqs.get(i).getSerial() < byId.getSerial()) {
                    break;
                }
            }
        } else {
            for (i = 0; i < faqs.size(); i++) {
                if (faqs.get(i).getSerial() > byId.getSerial()) {
                    break;
                }
            }
        }

        if (i == faqs.size()) {
            return RestResponse.ok(null, "1", "移动成功");
        }

        faq = faqs.get(i);


        if (byId.getId().equals(faq.getId())) {
            return RestResponse.ok(null, "1", "移动成功");
        }

        Integer serial = faq.getSerial();
        faq.setSerial(byId.getSerial());
        byId.setSerial(serial);

        this.updateById(faq);
        this.updateById(byId);

        return RestResponse.ok(null, "1", "移动成功");
    }
}